#ifndef _NES6502_H_
#define _NES6502_H_

#ifdef __cplusplus
extern "C" {
#endif /* __cplusplus */

#include "types.h"

/* Define this to enable decimal mode in ADC / SBC (not needed in NES) */
/*#define  NES6502_DECIMAL*/

/* number of bank pointers the CPU emulation core handles */
#ifdef NSF_PLAYER
#define  NES6502_4KBANKS
#endif

#ifdef NES6502_4KBANKS
#define  NES6502_NUMBANKS  16
#define  NES6502_BANKSHIFT 12
#else
#define  NES6502_NUMBANKS  8
#define  NES6502_BANKSHIFT 13
#endif

#define  NES6502_BANKSIZE  (0x10000 / NES6502_NUMBANKS)
#define  NES6502_BANKMASK  (NES6502_BANKSIZE - 1)

/* P (flag) register bitmasks */
#define  N_FLAG         0x80
#define  V_FLAG         0x40
#define  R_FLAG         0x20  /* Reserved, always 1 */
#define  B_FLAG         0x10
#define  D_FLAG         0x08
#define  I_FLAG         0x04
#define  Z_FLAG         0x02
#define  C_FLAG         0x01

/* Vector addresses */
#define  NMI_VECTOR     0xFFFA
#define  RESET_VECTOR   0xFFFC
#define  IRQ_VECTOR     0xFFFE

/* cycle counts for interrupts */
#define  INT_CYCLES     7
#define  RESET_CYCLES   6

#define  NMI_MASK       0x01
#define  IRQ_MASK       0x02

/* Stack is located on 6502 page 1 */
#define  STACK_OFFSET   0x0100

typedef uint8 (*nes6502_memread)(uint32 address);
typedef void (*nes6502_memwrite)(uint32 address, uint8 value);

typedef struct
{
  uint8 *mem_page[NES6502_NUMBANKS];  /* memory page pointers */
  
  uint32 pc_reg;
  
  uint8 a_reg, p_reg;
  uint8 x_reg, y_reg;
  uint8 s_reg, __padding;
  uint8 int_pending, jammed;
  
  int32 total_cycles, burn_cycles;
  
  nes6502_memread read_handler;
  nes6502_memwrite write_handler;
} nes6502_context;

/* Functions which govern the 6502's execution */
extern void nes6502_reset(void);
extern int nes6502_execute(int total_cycles);
extern void nes6502_nmi(void);
extern void nes6502_irq(void);
extern void nes6502_pending_irq(void);
extern uint8 nes6502_getbyte(uint32 address);
extern uint32 nes6502_getcycles();
extern void nes6502_burn(int cycles);

/* Context get/set */
extern void nes6502_set_zeropage();
extern nes6502_context *nes6502_get_current_context();
extern void nes6502_setcontext();

#ifdef __cplusplus
}
#endif /* __cplusplus */

#endif /* _NES6502_H_ */

/*
** nes6502.h,v
** Revision 1.1.2.4  2002/11/05 09:59:02  tekezo
** *** empty log message ***
**
** Revision 1.6  2001/10/08 17:39:33  tekezo
** fds pre-support
**
** Revision 1.5  2001/10/03 10:09:37  tekezo
** support save snapshot
**
** Revision 1.4  2001/09/23 17:28:48  tekezo
** bug fix for gradius2
**
** Revision 1.3  2001/09/23 10:53:54  tekezo
** now support SNSS
**
** Revision 1.2  2001/09/20 10:44:02  tekezo
** *** empty log message ***
**
** Revision 1.12  2000/09/08 11:54:48  matt
** optimize
**
** Revision 1.11  2000/09/07 21:58:18  matt
** api change for nes6502_burn, optimized core
**
** Revision 1.10  2000/09/07 01:34:55  matt
** nes6502_init deprecated, moved flag regs to separate vars
**
** Revision 1.9  2000/08/28 12:53:44  matt
** fixes for disassembler
**
** Revision 1.8  2000/08/28 01:46:15  matt
** moved some of them defines around, cleaned up jamming code
**
** Revision 1.7  2000/08/16 04:56:37  matt
** accurate CPU jamming, added dead page emulation
**
** Revision 1.6  2000/07/30 04:32:00  matt
** now emulates the NES frame IRQ
**
** Revision 1.5  2000/07/17 01:52:28  matt
** made sure last line of all source files is a newline
**
** Revision 1.4  2000/06/09 15:12:25  matt
** initial revision
**
*/
